/******************************************************************************/
/*!
 * @file   ex_setting.h
 * @brief  外部ヘッダー
 * @note
 * @date   作成日　2021年12月21日
 * @author Ver.03 Copyright (c) Mobile Create Co., Ltd. 2016 All rights reserved.
 */
/******************************************************************************/

#ifndef _exsetting_H_
#define _exsetting_H_


#include <zephyr.h>

#include "ex_common.h"      // ADD M.Taguchi 2023/4/13 830001-112 :設定データ保存箇所とは別セクターでもう一つの出荷検査フラグを管理する
#include "ex_flash.h"       // ADD D.Praveen 2023/04/17 830001-112 : 設定データ保存箇所とは別セクターでもう一つの出荷検査フラグを管理する

// ADD-S T.Itai 2022/09/14 FOTA用JSONをS3から取得
// プロダクト
#define IM830_PRODUCT	"mobi"
// #define IM830_PRODUCT	"bus"
// #define IM830_PRODUCT	"taxi"
// ADD-E T.Itai 2022/09/14 FOTA用JSONをS3から取得
// 顧客コード長
#define CUSTOMER_CD_LEN	10

// 顧客PW長
// #define CUSTOMER_PW_LEN	10
#define CUSTOMER_PW_LEN	15

// シリアル番号長
#define SERIAL_NO_LEN	16
// ハードVer長
#define HW_VER_LEN	16
// fwVer長
#define FW_VER_LEN	16
// ADD-S M.Taguchi 2022/10/26 830設定連動
// 設定ID長
#define SETTING_ID_LEN	40
// ADD-E M.Taguchi 2022/10/26 830設定連動

// 設定データflashバージョン
#define SETTING_FLASH_VER		1		// CHG M.Taguchi 2023/1/31
// ADD-S M.Taguchi 2023/1/31 830001-22 :【要望】設定連動パラメータに圏外動態履歴保存機能のON/OFFを追加
// プロダクト種別
#define PRODUCT_MOBI			0x01							// モバロケ
#define PRODUCT_BUS				0x02							// バス
#define PRODUCT_TAXI			0x03							// 新視令
// 機能有効無効設定ON/OFF
#define SET_OFF					0x00							// 機能OFF
#define SET_ON					0x01							// 機能ON
// ADD-E M.Taguchi 2023/1/31 830001-22 :【要望】設定連動パラメータに圏外動態履歴保存機能のON/OFFを追加

// ============ structure ============
// 初期化情報
typedef struct
{
	uint16_t	crc_num;			// CRCチェック
	uint16_t	flash_ver;			// フラッシュバージョン
	uint8_t		shipment_flg;		// 初期化フラグ					 ★★★ erase状態では、0xffと想定
	char		reserved[43];		// 予約（設定を追加したらリザーブのサイズを変更すること）
}SHIPMENT_INFO;

// モバロケ設定
typedef struct
{
	uint32_t	distance_issue;						// 距離発行間隔(m)
	uint32_t	time_issue;							// 時間発行間隔(s)
	uint8_t		temp_issue;							// 温湿度センサーデータ発行（0：モバロケ発行しない、1：発行する）	// CHG M.Taguchi 2023/2/1
	uint8_t		inertia_issue;						// 慣性データセンサ発行切替（0：モバロケ発行しない、1：発行する）	// CHG M.Taguchi 2023/2/1
	char		dummy1[2];
	uint32_t	mobile_id;							// モバロケID
	char		customer_cd[CUSTOMER_CD_LEN+1];		// 顧客コード
	char		dummy2;
	char		customer_pw[CUSTOMER_PW_LEN+1];		// 顧客パスワード
	char		reserved[48];						// 予約（設定を追加したらリザーブのサイズを変更すること）
}MLOCA_SETTING;


// GNSSフィルタ設定
typedef struct
{
	//uint32_t	hacc_filter;			// haccフィルタ値  // CHG D.Praveen 2023/04/21 830001-123 : pdop値を830Configからdeviceinformation.jsonに移す
	//uint32_t	pdop_filter;			// pdopフィルタ値  // CHG D.Praveen 2023/04/21 830001-123 : pdop値を830Configからdeviceinformation.jsonに移す
	uint32_t 	imu_filter;				// imuフィルタ値
	char		reserved[48];			// 予約（設定を追加したらリザーブのサイズを変更すること）
}GNSS_FILTER_SETTING;

// 端末情報
typedef struct
{
	char		serial_no[SERIAL_NO_LEN+1];		// シリアル番号					★★★ サイズは変更の可能性あり
#if 0 // DEL-S K.Saho 2023/05/09 DBGコマンドのファームウェアバージョンが正しく表示されてない
	uint8_t		hw_ver;							// ハードウェアバージョン
	uint8_t		fw_mjr_ver;						// ファームウェアメジャーバージョン
	uint8_t		fw_min_ver;						// ファームウェアマイナーバージョン
	uint8_t		fw_pat_ver;						// ファームウェアパッチバージョン
#endif // DEL-E K.Saho 2023/05/09 DBGコマンドのファームウェアバージョンが正しく表示されてない
	char		check_date[CHECK_DATE_LEN+1];	// 検査日時						★★★ サイズは変更の可能性あり
	// CHG K.Saho 2023/02/14 830001-53 : アプリとモデムFOTAが必ずセットで更新されない
	//char		dummy1[2];
	uint8_t		fw_rollback_flg;					// ファームウェアロールバックフラグ
	// CHG D.Praveen 2023/04/21 830001-123 : pdop値を830Configからdeviceinformation.jsonに移す
	// char 		log_level;						// CHG Y.Saisho 2023/4/13 830001-115:設定データ領域にログレベル追加
	uint32_t	mctest_result;					// 出荷検査結果格納用		// ADD T.Itai 2022/10/18 出荷検査対応
	char		reserved[48];					// 予約（設定を追加したらリザーブのサイズを変更すること）
}DEVICE_INFO;

// ロガー情報
typedef struct
{
	uint32_t	save_time;			// ログ保存周期(s)
	char		reserved[48];		// 予約（設定を追加したらリザーブのサイズを変更すること）
}LOGGER_INFO;

// ADD-S M.Taguchi 2022/10/21 830設定連動
// CHG-S M.Taguchi 2023/1/31 830001-22 :【要望】設定連動パラメータに圏外動態履歴保存機能のON/OFFを追加
// 機能有効無効設定
typedef struct
{
	char		setting_id[SETTING_ID_LEN];		// 設定ID
	uint8_t		product;						// プロダクト（1：モバロケ、2：バス、3：新視令）
	uint8_t		setting_reboot;					// 設定連動後自動リブート設定（0：リブートなし、1：リブートあり）
	uint8_t		nosignal_save;					// 圏外時動態保存（0：保存しない、1：保存する）
	uint8_t		device_volume1;					// 初回位置発行時効果音設定（0：鳴動しない、1：鳴動する）
	uint8_t		device_volume2;					// 圏外時効果音設定（0：鳴動しない、1：鳴動する）
	int16_t		device_volume3;					// GNSS未測位時効果音設定（0：鳴動しない、10～600(s)：鳴動確定時間）		// CHG M.Taguchi 2023/4/13 830001-110 :【設定連動】GNSS未測位音にマイナスの値を配信すると値がオーバーフローしている
	char		dummy1;
	// uint32_t	fota_reboot;					// アプリFOTA後自動リブート設定 // DEL K.Saho 2022/12/09 アプリFOTA後のリブート処理を削除
	char		reserved[24];					// 予約（設定を追加したらリザーブのサイズを変更すること）
}FUNCTION_SETTING;
// CHG-E M.Taguchi 2023/1/31 830001-22 :【要望】設定連動パラメータに圏外動態履歴保存機能のON/OFFを追加
// ADD-E M.Taguchi 2022/10/21 830設定連動

// ADD-S D.Praveen 2023/04/21 830001-123 : pdop値を830Configからdeviceinformation.jsonに移す
// PARAM INFO
typedef struct 
{
	char 			log_level;				// log level
	uint32_t	pdop_filter;			// pdopフィルタ値
	uint32_t	hacc_filter;			// haccフィルタ値
	uint32_t	vacc_filter;			// vaccフィルタ値
	char      ssl_expire_date[17]; // ssl expire date
	char 			ssl_update_date[17];  // ssl update date
	char 			reserved[48]; 			
}PARAM_INFO;
// ADD-E  D.Praveen 2023/04/21 830001-123 : pdop値を830Configからdeviceinformation.jsonに移す

// 設定データ
typedef struct
{
	SHIPMENT_INFO			shipment_info;				// 初期化情報
	MLOCA_SETTING			mloca_setting;				// モバロケ情報
	GNSS_FILTER_SETTING		gnss_filter_setting;		// gnssフィルタ
	DEVICE_INFO				device_info;				// 端末情報
	LOGGER_INFO				logger_info;				// ロガー情報
// ADD-S M.Taguchi 2022/10/21 830設定連動
	FUNCTION_SETTING		function_setting;			// 機能有効無効設定		// CHG M.Taguchi 2023/1/31 830001-22 :【要望】設定連動パラメータに圏外動態履歴保存機能のON/OFFを追加
// ADD-E M.Taguchi 2022/10/21 830設定連動
	// char					reserved[120];				// 予約（設定を追加したらリザーブのサイズを変更すること）
	PARAM_INFO 				param_info;  				// ADD  D.Praveen 2023/04/21 830001-123 : pdop値を830Configからdeviceinformation.jsonに移す
}FLASH_SETTING_DATA;


// ============ prptotype ============
extern void setting_init(void);
extern SHIPMENT_INFO setting_shipment_getter(void);		// ADD M.Taguchi 2022/10/20 830設定連動
extern MLOCA_SETTING setting_mloca_getter(void);
extern GNSS_FILTER_SETTING setting_gnss_filter_getter(void);
extern DEVICE_INFO setting_device_info_getter(void);
extern LOGGER_INFO setting_logger_info_getter(void);
extern FUNCTION_SETTING setting_function_set_getter(void);		// CHG M.Taguchi 2023/1/31 830001-22 :【要望】設定連動パラメータに圏外動態履歴保存機能のON/OFFを追加
extern FLASH_MCTEST_INFO setting_function_mctest_getter(void); // ADD D.Praveen 2023/04/17 830001-112 : 設定データ保存箇所とは別セクターでもう一つの出荷検査フラグを管理する
extern PARAM_INFO setting_function_param_info_getter(void);  	// ADD D.Praveen 2023/04/21 830001-123 : pdop値を830Configからdeviceinformation.jsonに移す
// int setting_data_erase(void); // DEL K.Saho 2023/03/09 Cppcheckで検出された未使用関数を削除
extern bool setting_mloca_distance_issue_update(int value);
extern bool setting_mloca_time_issue_update(int value);
// CHG-S M.Taguchi 2023/2/1
extern bool setting_mloca_temp_issue_update(int value);
extern bool setting_mloca_inertia_issue_update(int value);
// CHG-E M.Taguchi 2023/2/1
extern bool setting_mloca_customer_code_update(char *pCode, int len);
extern bool setting_mloca_customer_pw_update(char *pCode, int len);
extern bool setting_device_serialno_update(char *pSerial, int len);
// CHG-S D.Praveen 2023/04/21 830001-114 :ログレベル変更をawsのjsonファイルから行えるようにする
#if 0
extern bool setting_device_log_level_update(int pLevel); // ADD D.Praveen 2023/04/17 830001-114 :ログレベル変更をawsのjsonファイルから行えるようにする
#else
// CHG-S D.Praveen 2023/04/21 830001-123 : pdop値を830Configからdeviceinformation.jsonに移す
#if 0
extern bool setting_device_log_level_update(char pLevel);
#else
//extern bool setting_param_info_update(char log_level, uint32_t pdop_filter, uint32_t hacc_filter, uint32_t vacc_filter);
//extern bool setting_param_info_update(char log_level, uint32_t pdop_filter, uint32_t hacc_filter, uint32_t vacc_filter, char* ssl_expire_date);
extern bool setting_param_info_update(char log_level, uint32_t pdop_filter, uint32_t hacc_filter, uint32_t vacc_filter, char* ssl_expire_date, char* ssl_update_date);
#endif
// CHG-E D.Praveen 2023/04/21 830001-123 : pdop値を830Configからdeviceinformation.jsonに移す
#endif
// CHG-E D.Praveen 2023/04/21 830001-114 :ログレベル変更をawsのjsonファイルから行えるようにする
extern bool setting_mctest_result_update(uint32_t mctest_result);	// ADD-S T.Itai 2022/10/18 出荷検査対応
// ADD-S D.Praveen 2022/09/08 距離発行の判定に使用するpdop値の判定条件を厳しくすること
extern bool setting_gnss_pdop_update(int value);
// ADD-E D.Praveen 2022/09/08 距離発行の判定に使用するpdop値の判定条件を厳しくすること

// bool setting_device_hwver_update(uint8_t hw_ver);		// 未使用なのでコメントアウト
// bool setting_device_fwver_update(uint8_t mjr_ver, uint8_t min_ver, uint8_t pat_ver);		// 未使用なのでコメントアウト
extern bool setting_device_checkdate_update(char *pHwver, int len);
extern bool setting_logger_save_time_update(int value);
// bool setting_ota_update(struct rsp_license_setting_obj *license_setting_obj, char *license_settingid, int license_auto_reload);		// ADD M.Taguchi 2022/10/24 830設定連動
// ADD-S M.Taguchi 2023/2/3
extern bool setting_function_setting_reboot_update(int value);
extern bool setting_function_nosignal_save_update(int value);
extern bool setting_function_volume1_update(int value);
extern bool setting_function_volume2_update(int value);
extern bool setting_function_volume3_update(int value);
// ADD-E M.Taguchi 2023/2/3
#if(DCOM_MODEM_FOTA == 1)
extern bool get_fw_rollback_flg(void); // ADD-S K.Saho 2023/02/16 課題830001-53レビュー指摘内容の修正
extern bool setting_device_fw_rollback_flg(bool value); // ADD K.Saho 2023/02/14 830001-53 : アプリとモデムFOTAが必ずセットで更新されない
#endif // #if(DCOM_MODEM_FOTA == 1)
extern bool setting_mobiloca_init(void);
extern bool setting_gnss_filter_init(void);
extern bool setting_device_info_init(void);
extern bool setting_logger_info_init(void);
extern bool setting_function_setting_init(void);	// CHG M.Taguchi 2023/1/31 830001-22 :【要望】設定連動パラメータに圏外動態履歴保存機能のON/OFFを追加
// static void convertparam_flash(void);		// ADD M.Taguchi 2022/10/24 830設定連動


#endif //_exsetting_H_
/****************************************************************************************/
/* FILE END : ex_setting.h																*/
/****************************************************************************************/
